close all
clearvars -except list_

disp('Serbia data filtration ...')

%% dates
shist = qq(2007,1);
ehist = qq(2022,4);
rng_excovid = shist:ehist;
%% load data
disp('Serbia: Load data')
load('RS_BC.mat', 'dobs');
tmp = load('../EA/EA_BC.mat', 'dobs');

% take the EA data as the foreign block
dobs.OBS_RS_RW        = tmp.dobs.OBS_RS;
dobs.OBS_L_CPI_RW     = tmp.dobs.OBS_L_CPI;
dobs.OBS_L_GDP_RW_GAP = tmp.dobs.OBS_L_GDP_GAP;
dobs.OBS_RR_RW_BAR    = tmp.dobs.OBS_RR_BAR;

% expert tunes
dobs.OBS_D4L_CPI_TAR = hpf(dobs.OBS_L_CPI-dobs.OBS_L_CPI{-4}, Inf, 'level', Series(qq(2018,1):ehist, 3));

clear tmp
%% load model
disp('Serbia: Load and adjust the model')
run('../Model/readmodel.m')

%% estimate steady-state
p = get(m, 'parameters');

p.ss_D4L_CPI_TAR   = mean(dobs.OBS_L_CPI(rng_excovid) - dobs.OBS_L_CPI{-4}(rng_excovid));
p.ss_DLA_Z_BAR     = mean((dobs.OBS_L_S(rng_excovid) - dobs.OBS_L_S{-4}(rng_excovid)) + (dobs.OBS_L_CPI_RW(rng_excovid) - dobs.OBS_L_CPI_RW{-4}(rng_excovid)) ...
                     - (dobs.OBS_L_CPI(rng_excovid) - dobs.OBS_L_CPI{-4}(rng_excovid)));

% Calibrated wedge of growth in potential output
p.ss_DLA_GDP_BAR   = mean(dobs.OBS_L_GDP(rng_excovid) - dobs.OBS_L_GDP{-4}(rng_excovid));

%% set stds
p.std_SHK_L_GDP_GAP    = 0.5*std(hpf2(4*(dobs.OBS_L_GDP - dobs.OBS_L_GDP{-1})));
p.std_SHK_DLA_CPI      = 1/4*std(hpf2(4*(dobs.OBS_L_CPI - dobs.OBS_L_CPI{-1})));
p.std_SHK_L_S          = 1/2*std(hpf2(4*(dobs.OBS_L_S - dobs.OBS_L_S{-1})));
p.std_SHK_RS           = 2*std(hpf2(dobs.OBS_RS));
p.std_SHK_D4L_CPI_TAR  = 1/3*p.std_SHK_DLA_CPI;
p.std_SHK_DLA_Z_BAR    = 1/2*p.std_SHK_L_GDP_GAP;
p.std_SHK_DLA_GDP_BAR  = 1/5*p.std_SHK_L_GDP_GAP;

% EA case
p.std_SHK_L_GDP_RW_GAP = 1*std(dobs.OBS_L_GDP_RW_GAP);
p.std_SHK_RR_RW_BAR    = 1/2*std(dobs.OBS_RR_RW_BAR);
p.std_SHK_RS_RW        = 1/2*std(dobs.OBS_L_GDP_RW_GAP);
p.std_SHK_DLA_CPI_RW   = 2/3*std(dobs.OBS_L_GDP_RW_GAP);

% MP
p.std_SHK_DLA_S_TAR    = 0;
p.std_SHK_PREM         = 1.00;

std_of_shocks = get(m,'std');
list_std      = fieldnames(std_of_shocks);
mult          = dbbatch(std_of_shocks, '$0', 'Series(shist:ehist, 1)');

% high inflation shocks
mult.std_SHK_DLA_CPI(shist:ehist) = 1;
mult.std_SHK_DLA_CPI(qq(2021,1):ehist) = 2*3; 
mult.std_SHK_RS(shist:ehist) = 1;
mult.std_SHK_RS(qq(2010,1):ehist) = 2; 


m = assign(m, p);
m = solve(m);
m = sstate(m);

%% estimate parameters 
disp('Serbia: Estimation of parameters')

E.a2 = {NaN, 0, 2, distribution.InvGamma.fromMeanStd(m.a2, 1*m.a2)};
E.a3 = {NaN, 0, 1, distribution.Beta.fromMeanStd(m.a3, 1/2*m.a3)};
E.std_SHK_DLA_CPI = {NaN, 0, 15, distribution.InvGamma.fromMeanStd(p.std_SHK_DLA_CPI, 1*p.std_SHK_DLA_CPI)};

E.b1 = {NaN, 0, 1, distribution.Beta.fromMeanStd(m.b1, 1/5*m.b1)};%<<<---<<<<
E.b2 = {NaN, 0, 1, distribution.Gamma.fromMeanStd(m.b2, 1/2*m.b2)};%<<<---<<<<
E.b3 = {NaN, 0, 1, distribution.Gamma.fromMeanStd(m.b3, 1/2*m.b3)};
E.b4 = {NaN, 0, 1, distribution.Beta.fromMeanStd(m.b4, 1/4*m.b4)};

E.std_SHK_L_GDP_GAP = {NaN, 0, 15, distribution.InvGamma.fromMeanStd(p.std_SHK_L_GDP_GAP, 3*p.std_SHK_L_GDP_GAP)};%<<<---<<<<

E.c1 = {NaN, 0.8, 1.5, distribution.Gamma.fromMeanStd(m.c1, 5*1/15*m.c1)};
E.c2 = {NaN, 0.25, 0.75, distribution.Beta.fromMeanStd(m.c2, 5*1/15*m.c2)};


[est, pos, C, H, mest, v, delta, Pdelta] = ...
    estimate(m, dobs, shist:ehist, E, ...
    'filter', {'relative', true,'Multiply', mult}, 'optimSet', {'maxFunEvals', 10000, 'maxIter', 100}, 'noSolution', 'penalty' );
disp('Serbia: Estimated parameters')
est

m = mest;
m = sstate(m);
return
%%
N = 1e5;
[theta, logpost, ar] = arwm(pos, N, ...
    'progress=', true, 'adaptScale=', 2, 'adaptProposalCov=', 1, 'burnin=', 0.2); %'burnin=', 0.20

%%
E.c1 = {NaN, 0.2, 1.5, distribution.Gamma.fromMeanStd(m.c1, 5*1/15*m.c1)};
E.c2 = {NaN, 0, 0.75, distribution.Beta.fromMeanStd(m.c2, 5*1/15*m.c2)};

[pr, po, h] = plotpp(E, est, theta, 'PlotInit', false, 'PlotMode', false, 'PlotBounds', false, ...
    'KsDensity', 100, 'smoothfactor', [], 'subplot', [3,4]); 
legend('Prior Distribution','Posterior Distribution', ...
       'Box', 'off');
ftitle(h, 'Prior and Posterior Distributions');
 saveas( gcf , 'Bayes_RS' , 'mfig' )
 save mh.mat theta
 return
%% filtration with estimated model
disp('Serbia: Filtration using estimated model')

% std_of_shocks = get(m,'std');
% list_std      = fieldnames(std_of_shocks);
% mult          = dbbatch(std_of_shocks, '$0', 'Series(shist:ehist, 1)');
% 
% % high inflation shocks
% mult.std_SHK_DLA_CPI(shist:ehist) = 1;
% mult.std_SHK_DLA_CPI(qq(2021,1):ehist) = 3; 
% mult.std_SHK_RS(shist:ehist) = 1;
% mult.std_SHK_RS(qq(2021,1):ehist) = 3; 
% 
% m = solve(m);
% m = sstate(m);
[m, f, v, delta, pe] = filter(m, dobs, shist:ehist,'Multiply', mult);

% shocks' contributions to log-likelihood
eList = get(m,'eList');
disp('Contributions of shocks to the overall minus log likelihood');
for i=1:length(eList)
  s = f.mean.(eList{i});
  ss = sum((s/m.(['std_' eList{i}])).^2/v);
  disp(sprintf('%30s: %12.5g', eList{i}, ss));
end
disp(['Scalling: ' num2str(v)])

f = f.mean;
dbsave(f, 'KF_RS.csv');
save('outcomes_RS.mat', 'm', 'f')
return
%% Graphs
disp('Serbia: Reporting filtration results')

% Specify country
country = 'Serbia';

% Report
x = report.new(country,'visible',true);

% Figures
rng = shist:ehist;
sty = struct();
sty.line.linewidth = 1.5;
% sty.line.linestyle = {'-';'-';'-'};
sty.axes.box = 'off';
sty.legend.location = 'NorthWest';

% Output Gap
x.figure('Output Gap','subplot',[3,2],'style',sty,'range',rng,'dateformat','YY:P');

x.graph('GDP, 100*log','legend',false);
x.series('',[f.L_GDP f.L_GDP_BAR]);

x.graph('GDP Growth qoq, percent annualized','legend',true);
x.series({'Actual', 'Trend'},[f.DLA_GDP f.DLA_GDP_BAR]);

x.graph('Output Gap, percent','legend',false);
x.series('',[f.L_GDP_GAP]);

x.graph('Foreign GDP Gap, percent','legend',false);
x.series('',[f.L_GDP_RW_GAP]);

x.graph('MCI, percent','legend',true);
x.series('', [f.b4*f.RR_GAP, (1-f.b4)*(-f.L_Z_GAP)], 'legend', {'RR Gap', 'L Z GAP'}, 'plotfunc', @conbar);
x.series('MCI',[f.MCI]);

% Output gap decompostion          
x.figure('Output Gap Decomposition','subplot',[2,1],'style',sty,'range',rng,'dateformat','YY:P');

x.graph('Output Gap, percent','legend',true);
x.series('Actual', f.L_GDP_GAP);
x.series('Predicted', f.L_GDP_GAP-f.SHK_L_GDP_GAP);

x.graph('Output Gap Decomposition, pp','legend',true);
x.series('',[f.b1*f.L_GDP_GAP{-1}, -f.b2*f.b4*f.RR_GAP, f.b2*(1-f.b4)*f.L_Z_GAP, f.b3*f.L_GDP_RW_GAP, f.SHK_L_GDP_GAP], ... 
            'legend=', {'Lag', 'RIR Gap', 'RER Gap', 'RW Gap', 'Shock'}, 'plotfunc', @conbar);

% RIR and RER      
x.figure('Real Interest and Exchange Rates','subplot',[3,2],'style',sty,'range',rng,'dateformat','YY:P');

x.graph('Real Interest Rate, percent p.a.','legend',false);
x.series('',[f.RR f.RR_BAR m.RR_BAR]);

x.graph('Risk Premium, percent p.a.','legend',false);
x.series('',[f.PREM]);

x.graph('Real Exchange Rate, 100*log','legend',false);
x.series('',[f.L_Z f.L_Z_BAR]);

x.graph('Eq. Real Exchange Rate Apprec., percent','legend',false);
x.series('',[f.DLA_Z f.DLA_Z_BAR]);

x.graph('RER Decomposition, pp','legend',true);
x.series('',[f.DLA_S, -f.DLA_CPI, f.DLA_CPI_RW], ... 
            'legend=', {'Nom. ER', 'Domestic Infl.', 'RW Infl.'}, 'plotfunc', @conbar);

x.graph('RER Gap, percent','legend',false);
x.series('',[f.L_Z_GAP]);

x.figure('Natural Rate of Interest','subplot',[2,1],'style',sty,'range',rng,'dateformat','YY:P');

x.graph('Real Interest Rate, percent p.a.','legend',true);
x.series('',[f.RR f.RR_BAR], 'legend', {'Actual', 'Natural'});

x.graph('RIR Decomposition, pp','legend',true);
x.series('',[2*f.c1*f.c2*f.D4L_GDP_BAR, 2*f.c1*(1-f.c2)*f.D4L_Z_BAR], ... 
            'legend=', {'Potential Growth', 'RER Apprec.'}, 'plotfunc', @conbar);
x.series('',[f.RR_BAR]);


% Inflation
x.figure('Inflation','subplot',[2,2],'style',sty,'range',rng,'dateformat','YY:P');

x.graph('Inflation, percent','legend',true);
x.series('QoQ',[f.DLA_CPI]);
x.series('YoY',[f.D4L_CPI]);

x.graph('Inflation D4L_CPI_TAR, percent','legend',true);
x.series('YoY',[f.D4L_CPI]);
x.series('D4L\_CPI\_TAR',[f.D4L_CPI_TAR]);

x.graph('Marginal Cost, percent','legend',false);
x.series('',[f.RMC]);

x.graph('Marginal Cost Decomposition, pp','legend',true);
x.series('',[f.a3*f.L_GDP_GAP, (1-f.a3)*f.L_Z_GAP], 'legend=', {'Output Gap', 'RER Gap'}, 'plotfunc', @conbar);

% Inflation decompostion
x.figure('Inflation','subplot',[3,1],'style',sty,'range',rng,'dateformat','YY:P');

x.graph('Inflation qoq, percent','legend',true);
x.series('Actual',f.DLA_CPI);
x.series('Predicted',f.DLA_CPI-f.SHK_DLA_CPI);

x.graph('Inflation and Marginal Costs, percent','legend',true);
x.series('Inflation (de-mean)',f.DLA_CPI-mean(f.DLA_CPI));
x.series('RMC', f.RMC);

x.graph('Inflation Decomposition, pp','legend',true);
x.series('',[f.a1*f.DLA_CPI{-1}, (1-f.a1)*f.E_DLA_CPI, f.a2*f.a3*f.L_GDP_GAP, f.a2*(1-f.a3)*f.L_Z_GAP, f.SHK_DLA_CPI], ... 
            'legend=', {'Lag', 'Lead', 'Output Gap', 'RER Gap', 'Shock'}, 'plotfunc', @conbar);

% Interest rate decompostion          
x.figure('Reaction Function Decomposition','subplot',[2,1],'style',sty,'range',rng,'dateformat','YY:P');

x.graph('Nom. Interest Rate, percent','legend',true);
x.series('Actual', f.RS);
x.series('Predicted', f.RS-f.SHK_RS);

x.graph('Interest Rate - Reaction Function Decomposition, pp','legend',true);
x.series('',[f.g1*f.RS{-1}, (1-f.g1)*f.RSNEUTRAL, (1-f.g1)*f.g2*f.D4L_CPI_DEV, (1-f.g1)*f.g3*f.L_GDP_GAP, ...
            f.SHK_RS], ... 
            'legend=', {'Smoothing', 'Neutral Rate', 'Infl. Dev. from D4L\_CPI\_TAR', 'Output Gap', 'KZT Dev. from D4L\_CPI\_TAR', 'Shock'}, 'plotfunc', @conbar);

% Nominal exchange rate decomposition 
x.figure('Nom. Exchange Rate Target and Its Decomposition','subplot',[2,1],'style',sty,'range',rng,'dateformat','YY:P');

x.graph('Nom. Appreciation and Target, percent','legend',true);
x.series('Actual', f.DLA_S);
x.series('Target', f.DLA_S_TAR);

x.figure('Foreign Variables', 'subplot', [3,2], 'style', sty, 'range', rng);

x.graph('Foreign Nom. Interest Rate, % pa','legend',false);
x.series('',[f.RS_RW]);

x.graph('Foreign Real Rate, %','legend',true);
x.series({'Level', 'Trend'},[f.RR_RW, f.RR_RW_BAR]);

x.graph('Foreign CPI Inflation, %','legend', false);
x.series({'QoQ', 'YoY'},[f.DLA_CPI_RW, f.D4L_CPI_RW]);

x.graph('Foreign Output Gap, %','legend',false);
x.series('',[f.L_GDP_RW_GAP]);

x.pagebreak();

% Shocks
x.figure('Structural Shocks','subplot',[3,3],'style',sty,'range',rng,'dateformat','YY:P');

x.graph('Inflation','legend',false);
x.series('',[f.SHK_DLA_CPI]);

x.graph('Output Gap','legend',false);
x.series('',[f.SHK_L_GDP_GAP]);

x.graph('Interest Rate','legend',false);
x.series('',[f.SHK_RS]);

x.graph('Exchange Rate','legend',false);
x.series('',[f.SHK_L_S]);


%% Publishing
x.publish('Filtration_RS','display',false);
disp('Done!!!');
